from org.maemo.hermes.engine.friend import Friend
from org.maemo.hermes.engine.contact import Contact
import evolution

class Hermes:
    """Encapsulate the process of syncing online friends' information with the
       Evolution contacts' database. This should be used as follows:
       
         * Initialise, passing in a GUI callback.
         * Call Syncjob.
         * Retrieve information on changes effected.
         * Call update_contact to enact manual mapping.
         
       Copyright (c) Andrew Flegg <andrew@bleb.org> 2010.
       Released under the Artistic Licence."""

    
    # -----------------------------------------------------------------------
    def __init__(self, services, gui_progress):
        """Constructor. Passed a list of services, and a callback method
           which must implement the following API.
                              
             progress(i, j) - the application is currently processing friend 'i' of
                              'j'. Should be used to provide the user a progress bar.
        """
        
        # -- These fields are currently part of the API...
        #
        self.updated = []
        self.matched = []
        self.unmatched = []
        self.friends = {}
        self.addresses = None
        
        # -- Other initialisation...
        #
        self._services = services
        self._progress = gui_progress
    
    
    # -----------------------------------------------------------------------
    def run(self, resync=False):
        """Load information on the authenticated user's friends. Synchronise Facebook
           profiles to contact database. If resync is false, no existing information
           will be overwritten."""

        class FakeContact():
            def get_name(self):
                return "Fredrik Wendt"
            def get_emails(self):
                return ["fredrik@wendt.se", "maemohermes@wendt.se"]
            def get_photo(self):
                return None
            def get_mapped_to(self):
                return set(["facebook", "gravatar"])
        self.matched = [FakeContact()]
 
#        self._sync_job = SyncJob(services, [FakeContact()], self.progress)
#        self._sync_job.run()
#        self._sync_job.get_unmatched_friends()
#        self._sync_job.get_updated_contacts()
#        self._sync_job.get_matched_contacts()
        pass
    
    
    # -----------------------------------------------------------------------
    def run_alt(self, overwrite_existing_fields=False):
        contacts = []
        self.addresses = evolution.ebook.open_addressbook('default')
        for contact in self.addresses.get_all_contacts():
            contacts.append(Contact(self.addresses, contact))

        # warm up
        for service in self._services:
            print "pre-process:", service.get_id()
            for contact in contacts:
                service.pre_process_contact(contact)
                
        # fetch data
        for service in self._services:
            print "process_friends:", service.get_id()
            service.process_friends()
        
        # combine results into one friend
        for contact in contacts:
            result = Friend()
            for service in self._services:
                print "process_contact:", service.get_id()
                friend = service.process_contact(contact)
                if friend:
                    contact.add_mapping(service.get_id())
                    friend.decorate(result)
            
            if result.get_name() is not None:
                self.update_contact(result, overwrite_existing_fields)
            else:
                self.unmatched.add(contact)
            
        # give services a chance to create new contacts
        for service in self._services:
            print "create_contacts:", service.get_id()
            for friend in service.create_contacts():
                self.create_contact_from_friend(friend)
                
        # finalisation
        for service in self._services:
            print "finalize:", service.get_id()
            service.finalise(self.updated, overwrite_existing_fields)
            
        # commit changes
        for contact in self.updated:
            print "committing changes to:", contact.get_name(), contact
            self.addresses.commit_contact(contact.get_econtact())
        

    # -----------------------------------------------------------------------
    def update_contact(self, contact, friend, resync=False):
        """Update the given contact with information from the given friend."""
        
        print "updating contact ", contact, " with friend ", friend
        self.updated.append(contact)
        self.matched.append(contact)
        if friend.get_source() is not None:
            contact.add_mapping(friend.get_source())


    # -----------------------------------------------------------------------
    def create_contact_from_friend(self, friend):
        econtact = evolution.ebook.EContact()
        econtact.props.full_name = friend['name']
        econtact.props.given_name = friend['first_name']
        econtact.props.family_name = friend['last_name']
        contact = Contact(self.addresses, econtact)
                
        self.addresses.add_contact(contact)
        self.update_contact(contact, friend)
        
        print "Created [%s]" % (contact.get_name())
